@@ -423,6 +423,7 @@ class webEngine(QWebEngineView):
         # self.showMaximized()
 
     def addRecent(self, path):
+        os.makedirs(os.path.join(os.path.expanduser("~"), ".tissuumaps"), exist_ok=True)
         recentFile = os.path.join(
             os.path.expanduser("~"), ".tissuumaps", "recents.json"
         )
@@ -534,7 +535,7 @@ class webEngine(QWebEngineView):
             except:
                 pass
 
-            logging.error("Impossible to load", self.location)
+            logging.error("Impossible to load: " + self.location)
             time.sleep(0.1)
         if len(self.args) > 0:
             if not self.openImagePath(os.path.abspath(self.args[0])):
